@@ -63,7 +63,7 @@
 
 #define OPTARGS_CHECK_GET(wrong,right) lokke==argc-1?(fprintf(stderr,"Must supply argument for '%s'\n",argv[lokke]),exit(-4),wrong):right
 
-#define OPTARGS_BEGIN(das_usage) {int lokke;const char *usage=das_usage;for(lokke=1;lokke<argc;lokke++){char *a=argv[lokke];if(!strcmp("--help",a)||!strcmp("-h",a)){fprintf(stderr,usage);exit(0);
+#define OPTARGS_BEGIN(das_usage) {int lokke;const char *usage=das_usage;for(lokke=1;lokke<argc;lokke++){char *a=argv[lokke];if(!strcmp("--help",a)||!strcmp("-h",a)){fprintf(stderr,"%s",usage);exit(0);
 #define OPTARG(name,name2) }}else if(!strcmp(name,a)||!strcmp(name2,a)){{
 #define OPTARG_GETINT() OPTARGS_CHECK_GET(0,atoi(argv[++lokke]))
 //int optargs_inttemp;
@@ -72,7 +72,7 @@
 #define OPTARG_GETSTRING() OPTARGS_CHECK_GET("",argv[++lokke])
 #define OPTARG_LAST() }}else if(lokke==argc-1 && argv[lokke][0]!='-'){lokke--;{
 #define OPTARGS_ELSE() }else if(1){
-#define OPTARGS_END }else{fprintf(stderr,usage);exit(-1);}}}
+#define OPTARGS_END }else{fprintf(stderr,"%s",usage);exit(-1);}}}
 
 
 
@@ -83,7 +83,7 @@ static float min_buffer_time=-1.0f;
 static float max_buffer_time=40.0f;
 static jack_client_t *client=NULL;
 #define DEFAULT_NUM_CHANNELS 2
-static int channels=-1;
+static int num_channels=-1;
 static int bitdepth=0;
 static char *base_filename=NULL;
 static char *filename=NULL;
@@ -101,6 +101,7 @@ static char *filename_prefix="jack_capture_";
 static int64_t num_frames_recorded=0;
 static int64_t num_frames_to_record=0;
 static bool wait_for_keyboard=true;
+static bool no_stdin=false;
 static double recording_time=0.0;
 static const int disk_error_stop=0;
 static int disk_errors=0;
@@ -117,7 +118,7 @@ static bool write_to_mp3 = false;
 static int das_lame_quality = 2; // 0 best, 9 worst.
 static int das_lame_bitrate = -1;
 static bool use_jack_transport = false;
-
+static bool use_manual_connections = false;
 
 /* JACK data */
 static jack_port_t **ports;
@@ -129,6 +130,8 @@ static bool jack_has_been_shut_down=false;
 static int64_t unreported_overruns=0;
 static int total_overruns=0;
 
+static volatile int freewheel_mode=0;
+
 
 /* Disk thread */
 #if HAVE_LAME
@@ -205,8 +208,8 @@ static void verbose_print(const char *fmt, ...){
 }
 
 static void* my_calloc(size_t size1,size_t size2){
-  size_t size=size1*size2;
-  void* ret=malloc(size);
+  size_t size = size1*size2;
+  void*  ret  = malloc(size);
   if(ret==NULL){
     fprintf(stderr,"\nOut of memory. Try a smaller buffer.\n");
     return NULL;
@@ -217,7 +220,7 @@ static void* my_calloc(size_t size1,size_t size2){
 
 static bool set_high_priority(void){
   static bool shown_warning = false;
-  static int prio=-20;
+  static int  prio          = -20;
 
   while(prio<0 && setpriority(PRIO_PROCESS,0,prio)==-1)
     prio++;
@@ -271,8 +274,8 @@ static int buffer_size_in_bytes;
 // buffer size in bytes = block size in bytes * num_channels
 
 
-static int seconds_to_frames(float seconds){
-  return (int)(seconds*jack_samplerate);
+static int64_t seconds_to_frames(float seconds){
+  return (int64_t) (((long double)seconds)*((long double)jack_samplerate));
 }
 
 /*
@@ -347,7 +350,7 @@ static void buffers_init(){
   vringbuffer_set_autoincrease_callback(vringbuffer,autoincrease_callback,0);
 
   current_buffer = vringbuffer_get_writing(vringbuffer);
-  empty_buffer   = my_calloc(sizeof(sample_t),block_size*channels);
+  empty_buffer   = my_calloc(sizeof(sample_t),block_size*num_channels);
 }
 
 
@@ -355,10 +358,10 @@ static void buffers_init(){
 /////////////////////////////////////////////////////////////////////
 //////////////////////// PORTNAMES //////////////////////////////////
 /////////////////////////////////////////////////////////////////////
-static char **cportnames=NULL;
+static const char **cportnames=NULL;
 static int num_cportnames=0;
 
-static int findnumports(char **ports){
+static int findnumports(const char **ports){
   int ret=0;
   while(ports && ports[ret]!=NULL)
     ret++;
@@ -368,52 +371,52 @@ static int findnumports(char **ports){
 
 static void portnames_add_defaults(void){
   if(cportnames==NULL){
-    cportnames=(char **)jack_get_ports(client,NULL,NULL,JackPortIsPhysical|JackPortIsInput);
+    cportnames=jack_get_ports(client,NULL,NULL,JackPortIsPhysical|JackPortIsInput);
     num_cportnames=JC_MAX(DEFAULT_NUM_CHANNELS,findnumports(cportnames));
     if(num_cportnames==0){
       fprintf(stderr,"No physical output ports found in your jack setup. Exiting.\n");
       exit(0);
     }
-    if(channels==-1)
-      channels=DEFAULT_NUM_CHANNELS;
+    if(num_channels==-1)
+      num_channels=DEFAULT_NUM_CHANNELS;
   }else
-    if(channels==-1)
-      channels=num_cportnames;
+    if(num_channels==-1)
+      num_channels=num_cportnames;
 
-  if(channels<=0){
+  if(num_channels<=0){
     fprintf(stderr,"No point recording 0 channels. Exiting.\n");
     exit(0);
   }
 
-  // At this point, the variable "channels" has a known and valid value.
-  vu_vals=my_calloc(sizeof(float),channels);
-  vu_times=my_calloc(sizeof(int),channels);
-  vu_peaks=my_calloc(sizeof(int),channels);
-  vu_peakvals=my_calloc(sizeof(float),channels);
+  // At this point, the variable "num_channels" has a known and valid value.
+  vu_vals     = my_calloc(sizeof(float),num_channels);
+  vu_times    = my_calloc(sizeof(int),num_channels);
+  vu_peaks    = my_calloc(sizeof(int),num_channels);
+  vu_peakvals = my_calloc(sizeof(float),num_channels);
 
-  buffer_size_in_bytes = ALIGN_UP_DOUBLE(sizeof(buffer_t) + block_size*channels*sizeof(sample_t));
+  buffer_size_in_bytes = ALIGN_UP_DOUBLE(sizeof(buffer_t) + block_size*num_channels*sizeof(sample_t));
   verbose_print("buf_size_in_bytes: %d\n",buffer_size_in_bytes);
 }
 
 
 static void portnames_add(char *name){
-  char **new_outportnames;
+  const char **new_outportnames;
   int add_ch;
 
   if(name[strlen(name)-1]=='*'){
     char pattern[strlen(name)];
 
-    sprintf(pattern,name);
+    sprintf("%s",pattern,name);
 
     pattern[strlen(name)-1]=0;
 
-    new_outportnames=(char**)jack_get_ports(client,pattern,"",0);
-    //char **new_outportnames=(char**)jack_get_ports(client,"system:capture_1$","",0);
-    add_ch=findnumports(new_outportnames);
+    new_outportnames          = jack_get_ports(client,pattern,"",0);
+    //char **new_outportnames = (char**)jack_get_ports(client,"system:capture_1$","",0);
+    add_ch                    = findnumports(new_outportnames);
   }else{
-    new_outportnames=my_calloc(1,sizeof(char*));
-    new_outportnames[0]=name;
-    add_ch=1;
+    new_outportnames          = my_calloc(1,sizeof(char*));
+    new_outportnames[0]       = name;
+    add_ch                    = 1;
   }
 
   if(add_ch>0){
@@ -428,7 +431,7 @@ static void portnames_add(char *name){
     }
     
   }else{
-    fprintf(stderr,"\nWarning, No port(s) with name \"%s\".\n",name);
+    fprintf(stderr,"\nWarning, no port(s) with name \"%s\".\n",name);
     if(cportnames==NULL)
       if(silent==false)
 	fprintf(stderr,"This could lead to using default ports instead.\n");
@@ -436,12 +439,12 @@ static void portnames_add(char *name){
 }
 
 
-static char **portnames_get_connections(int ch){
+static const char **portnames_get_connections(int ch){
   if(ch>=num_cportnames)
     return NULL;
   else{      
-    jack_port_t* port=jack_port_by_name(client,cportnames[ch]);
-    char **ret;
+    jack_port_t  *port = jack_port_by_name(client,cportnames[ch]);
+    const char  **ret;
 
     if(port==NULL){
       print_message("Error, port with name \"%s\" not found.\n",cportnames[ch]);
@@ -449,10 +452,10 @@ static char **portnames_get_connections(int ch){
     }
 
     if(jack_port_flags(port) & JackPortIsInput){
-      ret=(char**)jack_port_get_all_connections(client,port);
+      ret    = jack_port_get_all_connections(client,port);
     }else{
-      ret=my_calloc(2,sizeof(char*));
-      ret[0]=cportnames[ch];
+      ret    = my_calloc(2,sizeof(char*));
+      ret[0] = cportnames[ch];
     }
     
     return ret;
@@ -496,6 +499,15 @@ static int iec_scale(float db) {
          return (int)(def * 2.0f);
 }
 
+static void msleep(int n){
+  usleep(n*1000);
+}
+
+static void print_ln(void){
+  putchar('\n');
+  //msleep(3);
+}
+
 static void print_console_top(void){
   if(use_vu){
     int lokke=0;
@@ -507,45 +519,46 @@ static void print_console_top(void){
     printf("   |");
     for(lokke=0;lokke<vu_len;lokke++)
       putchar(c);
-    printf("|\n");
+    printf("|");print_ln();
   }else{
-    printf("\n");
+    //print_ln();
   }
 }
 
 static void init_vu(void){
-  //int channels=4;
+  //int num_channels=4;
   int ch;
-  for(ch=0;ch<channels;ch++)
-    printf("\n");
+  for(ch=0;ch<num_channels;ch++)
+    print_ln();
 }
 
 static void init_show_bufferusage(void){
-  printf("\n");
+  print_ln();
 }
 
 
 static void move_cursor_to_top(void){
   printf("%c[%dA",0x1b,
          use_vu&&show_bufferusage
-         ? channels+1
+         ? num_channels+1
          : use_vu
-           ? channels
+           ? num_channels
            : show_bufferusage
              ? 1
              : 0);
+  msleep(3);
 }
 
 // Console colors:
 // http://www.linuxjournal.com/article/8603
 
 static void print_console(bool move_cursor_to_top_doit,bool force_update){
-  //int channels=4;
+  //int num_channels=4;
   int ch;
   char vol[vu_len+50];
-  vol[2]=':';
-  vol[3]='|';
-  vol[4+vu_len+1]=0;
+  vol[2]          = ':';
+  vol[3]          = '|';
+  vol[4+vu_len+1] = 0;
 
   // Values have not been updated since last time. Return.
   if(force_update==false && vu_vals[0]==-1.0f)
@@ -559,41 +572,41 @@ static void print_console(bool move_cursor_to_top_doit,bool force_update){
     // Set cyan color
     printf("%c[36m",0x1b);
 
-    for(ch=0;ch<channels;ch++){
-      int i;
-      float val=vu_vals[ch];
-      int pos;
-      vu_vals[ch]=-1.0f;
+    for(ch=0;ch<num_channels;ch++){
+      int   i;
+      float val   = vu_vals[ch];
+      int   pos;
+      vu_vals[ch] = -1.0f;
       
       if(vu_dB)
-        pos=iec_scale(20.0f * log10f(val * vu_bias)) * (vu_len) / 200;
+        pos = iec_scale(20.0f * log10f(val * vu_bias)) * (vu_len) / 200;
       else
-        pos=val*(vu_len);
+        pos = val*(vu_len);
       
       if (pos > vu_peaks[ch]) {
-        vu_peaks[ch] = pos;
+        vu_peaks[ch]    = pos;
         vu_peakvals[ch] = val;
-        vu_times[ch] = 0;
+        vu_times[ch]    = 0;
       } else if (vu_times[ch]++ > 40) {
-        vu_peaks[ch] = pos;
+        vu_peaks[ch]    = pos;
         vu_peakvals[ch] = val;
       }
       
       if(ch>9){
-        vol[0]='0'+ch/10;
-        vol[1]='0'+ch-(10*(ch/10));
+        vol[0] = '0'+ch/10;
+        vol[1] = '0'+ch-(10*(ch/10));
       }else{
-        vol[0]='0';
-        vol[1]='0'+ch;
+        vol[0] = '0';
+        vol[1] = '0'+ch;
       }
       
       for(i=0;i<vu_len;i++)
         if(vu_peaks[ch]==i && vu_peakvals[ch]>0.0f)
-          vol[4+i]='*';
+          vol[4+i] = '*';
         else if(i<=pos && val>0.0f)
-          vol[4+i]='-';
+          vol[4+i] = '-';
         else
-          vol[4+i]=' ';
+          vol[4+i] = ' ';
       
       if(vu_peakvals[ch]>=1.0f){
         vol[4+vu_len]='!';
@@ -615,13 +628,14 @@ static void print_console(bool move_cursor_to_top_doit,bool force_update){
     printf("%c[32m",0x1b); // green color
     printf("Buffer: %.2fs. / %.2fs. "
            "Disk high priority: [%c]. "
-           "Overruns: %d%c[0m\n",
+           "Overruns: %d%c[0m",
            //fmaxf(0.0f,buflen-bufleft),buflen,
            bufleft,buflen,
            disk_thread_has_high_priority?'x':' ',
            total_overruns,
            0x1b // reset color
            );
+    print_ln();
   }else{
     printf("%c[0m",0x1b); // reset colors
     fprintf(stderr,"%c[0m",0x1b); // reset colors
@@ -661,18 +675,23 @@ static void *helper_thread_func(void *arg){
     if(message_string[0]!=0){
       if(use_vu || show_bufferusage){
 	move_cursor_to_top();
+        if(!use_vu){
+          print_ln();
+        }
 	printf("%c[%dA",0x1b,1); // move up yet another line.
+        msleep(5);
 	printf("%c[31m",0x1b);   // set red color
 	{ // clear line
 	  int lokke;
 	  for(lokke=0;lokke<vu_len+5;lokke++)
 	    putchar(' ');
-	  putchar('\n');
+          print_ln();
 	  printf("%c[%dA",0x1b,1); // move up again
+          msleep(5);
 	}
       }
       printf(MESSAGE_PREFIX); 
-      printf(message_string);
+      printf("%s",message_string);
       message_string[0]=0;
       move_cursor_to_top_doit=false;
       if(use_vu || show_bufferusage)
@@ -685,17 +704,20 @@ static void *helper_thread_func(void *arg){
     if(init_meterbridge_ports()==1 && use_vu==false && show_bufferusage==false) // Note, init_meterbridge_ports will usually exit at the top of the function, where it tests for ports_meterbridge!=NULL (this stuff needs to be handled better)
       break;
 
-    usleep(1000000/20);
+    msleep(1000/20);
 
   }while(is_helper_running);
 
 
-  if(use_vu || show_bufferusage)
+  if(use_vu || show_bufferusage){
     print_console(true,true);
+  }
 
+  message_string[0]     = 0;
+  helper_thread_running = 0;
+
+  msleep(4);
 
-  message_string[0]=0;
-  helper_thread_running=0;
   return NULL;
 }
 
@@ -704,25 +726,29 @@ static void *helper_thread_func(void *arg){
 static pthread_mutex_t print_message_mutex = PTHREAD_MUTEX_INITIALIZER;  
 static void print_message(const char *fmt, ...){
   
-  if(helper_thread_running==0){
+  if(helper_thread_running==0 || write_to_stdout==true){
     va_list argp;
     va_start(argp,fmt);
     fprintf(stderr,"%c[31m" MESSAGE_PREFIX,0x1b);   // set red color
     vfprintf(stderr,fmt,argp);
-    fprintf(stderr,"%c[0m",0x1b); // reset colors
+    //fprintf(stderr,"%c[0m",0x1b); // reset colors
+    //msleep(1);
     va_end(argp);
   }else{
 
     pthread_mutex_lock(&print_message_mutex);{
 
       while(message_string[0]!=0)
-        usleep(20);
+        msleep(2);
     
       va_list argp;
       va_start(argp,fmt);
       vsprintf(message_string,fmt,argp);
       va_end(argp);
 
+      while(message_string[0]!=0)
+        msleep(2);
+
     }pthread_mutex_unlock(&print_message_mutex);
   }
 
@@ -730,16 +756,24 @@ static void print_message(const char *fmt, ...){
 
 
 void setup_helper_thread (void){
-  pthread_create(&helper_thread, NULL, helper_thread_func, NULL);
+  if(write_to_stdout==false){
+    pthread_create(&helper_thread, NULL, helper_thread_func, NULL);
+  }
 }
 
 static void stop_helper_thread(void){
   //helper_thread_running=0;
   is_helper_running=false;
-  pthread_join(helper_thread, NULL);
-      
-  if(use_vu||show_bufferusage)
+  if(write_to_stdout==false){
+    pthread_join(helper_thread, NULL);
+  }
+
+  /*
+  if(use_vu||show_bufferusage){
     printf("%c[0m",0x1b); // reset colors
+    usleep(1000000/2); // wait for terminal    
+  }
+  */
 }
 
 
@@ -766,8 +800,8 @@ static int mp3bufsize;
 static int open_mp3file(void){
   buffer_interleaved = false;
 
-  if(channels!=2){
-    print_message("Error. 2 channels supported for mp3 files only. (%d)\n",channels);
+  if(num_channels!=2){
+    print_message("Error. 2 channels supported for mp3 files only. (%d)\n",num_channels);
     return 0;
   }
 
@@ -816,7 +850,7 @@ static int open_mp3file(void){
 static int open_soundfile(void){
   int subformat;
 
-  SF_INFO sf_info={0};
+  SF_INFO sf_info; memset(&sf_info,0,sizeof(sf_info));
 
   if(write_to_stdout==true)
     return 1;
@@ -837,11 +871,11 @@ static int open_soundfile(void){
   /////////////////////////
 
   sf_info.samplerate = jack_get_sample_rate (client);
-  sf_info.channels = channels;
+  sf_info.channels   = num_channels;
   
   {
     int format=getformat(soundfile_format);
-    if(format==-1 && channels>2){
+    if(format==-1 && num_channels>2){
       fprintf(stderr,"Warning, the format \"%s\" is not supported. Using %s instead.\n",soundfile_format_multi,soundfile_format);
       sf_info.format=MORE_THAN_TWO_CHANNELS_FORMAT;
     }else if(format==-1){
@@ -930,7 +964,7 @@ static void close_soundfile(void){
 //#define UINT32_MAX 100000+(1024*1024)
 
 static int handle_filelimit(size_t frames){
-  int new_bytes=frames*bytes_per_frame*channels;
+  int new_bytes=frames*bytes_per_frame*num_channels;
 
   if(is_using_wav && (disksize + ((int64_t)new_bytes) >= UINT32_MAX-(1024*1024))){ // (1024*1024) should be enough for the header.
 
@@ -956,12 +990,13 @@ static int handle_filelimit(size_t frames){
 }
 
 
+
 // stdout_write made by looking at http://mir.dnsalias.com/oss/jackstdout/start
 // made by Robin Gareus.
 static int stdout_write(sample_t *buffer,size_t frames){
   static char *tobuffer=NULL;
   static int bufferlen=0;
-  int bytes_to_write=frames*channels*2;
+  int bytes_to_write=frames*num_channels*2;
 
   if(bufferlen<bytes_to_write){
     free(tobuffer);
@@ -970,9 +1005,9 @@ static int stdout_write(sample_t *buffer,size_t frames){
   }
 
   {
-    int i;
+    unsigned int i;
     int writeplace=0;
-    for(i=0;i<frames*channels;i++){
+    for(i=0;i<frames*num_channels;i++){
       int d = (int) rint(buffer[i]*32767.0);
       tobuffer[writeplace++] = (unsigned char) (d&0xff);
       tobuffer[writeplace++] = (unsigned char) (((d&0xff00)>>8)&0xff);
@@ -980,14 +1015,17 @@ static int stdout_write(sample_t *buffer,size_t frames){
   }
 
   {
-    int fd=fileno(stdout);
+    int   fd           = fileno(stdout);
+    char *tobuffer_use = tobuffer;
+
     while(bytes_to_write > 0){
-      int written=write(fd,tobuffer,bytes_to_write);
+      int written=write(fd,tobuffer_use,bytes_to_write);
       if(written==-1){
 	fprintf(stderr,"Error writing to stdout.\n");
 	break;
       }
       bytes_to_write -= written;
+      tobuffer_use   += written;
     }
   }
 
@@ -1128,12 +1166,12 @@ static void send_buffer_to_disk_thread(buffer_t *buffer){
 
 static void process_fill_buffer(sample_t *in[],buffer_t *buffer,int i,int end){
   sample_t *data=buffer->data;
-  int pos=buffer->pos*channels;
+  int pos=buffer->pos*num_channels;
   int ch;
 
   if(buffer_interleaved == true){
     for(;i<end;i++){
-      for(ch=0;ch<channels;ch++){
+      for(ch=0;ch<num_channels;ch++){
         sample_t val=in[ch][i];
         data[pos++]=val;
         val=fabsf(val);
@@ -1143,7 +1181,7 @@ static void process_fill_buffer(sample_t *in[],buffer_t *buffer,int i,int end){
     }
   }else{
     int start_i = i;
-    for(ch=0;ch<channels;ch++){
+    for(ch=0;ch<num_channels;ch++){
       sample_t *curr_in=in[ch];
       float max_vu=vu_vals[ch];
       for(i = start_i; i<end ; i++){
@@ -1156,8 +1194,8 @@ static void process_fill_buffer(sample_t *in[],buffer_t *buffer,int i,int end){
       vu_vals[ch] = max_vu;
     }
   }
-  //fprintf(stderr,"pos: %d %d\n",pos,channels);
-  buffer->pos=pos/channels;
+  //fprintf(stderr,"pos: %d %d\n",pos,num_channels);
+  buffer->pos=pos/num_channels;
 }
 
 static bool process_new_current_buffer(int frames_left){
@@ -1172,10 +1210,10 @@ static bool process_new_current_buffer(int frames_left){
 }
 
 static void process_fill_buffers(int jack_block_size){
-  sample_t *in[channels];
+  sample_t *in[num_channels];
   int i=0,ch;
 
-  for(ch=0;ch<channels;ch++)
+  for(ch=0;ch<num_channels;ch++)
     in[ch]=jack_port_get_buffer(ports[ch],jack_block_size);
       
   if(current_buffer==NULL && process_new_current_buffer(jack_block_size)==false)
@@ -1287,13 +1325,13 @@ static void start_meterbridge(int num_channels){
   meterbridge_pid=fork();
   if(meterbridge_pid==0){
     char *argv[100+num_channels];
-    argv[0]="meterbridge";
-    argv[1]="-t";
-    argv[2]=meterbridge_type;
-    argv[3]="-n";
-    argv[4]=meterbridge_jackname;
-    argv[5]="-r";
-    argv[6]=meterbridge_reference;
+    argv[0] = "meterbridge";
+    argv[1] = "-t";
+    argv[2] = meterbridge_type;
+    argv[3] = "-n";
+    argv[4] = meterbridge_jackname;
+    argv[5] = "-r";
+    argv[6] = meterbridge_reference;
     {
       int ch;
       for(ch=0;ch<num_channels;ch++){
@@ -1340,12 +1378,12 @@ static int init_meterbridge_ports(){
     }
 
     {
-      jack_port_t **ports_meterbridge2 = (jack_port_t **) my_calloc (sizeof (jack_port_t *),channels);  
+      jack_port_t **ports_meterbridge2 = (jack_port_t **) my_calloc (sizeof (jack_port_t *),num_channels);  
 
       ports_meterbridge2[0]=port1;
       {
 	int ch;
-	for(ch=1;ch<channels;ch++){
+	for(ch=1;ch<num_channels;ch++){
 	  sprintf(portname,"%s:meter_%d",meterbridge_jackname,ch+1);
 	  ports_meterbridge2[ch]=jack_port_by_name(client,portname);
 	  if(ports_meterbridge2[ch]==NULL){
@@ -1379,13 +1417,13 @@ static int compare(const void *a, const void *b){
 
 static int reconnect_ports_questionmark(void){
   int ch;
-  int ret=0;
-  for(ch=0;ch<channels;ch++){
-    char **connections1 = portnames_get_connections(ch);
+
+  for(ch=0;ch<num_channels;ch++){
+    const char **connections1 = portnames_get_connections(ch);
     const char **connections2 = jack_port_get_all_connections(client,ports[ch]);
 
-    int memb1=findnumports(connections1);
-    int memb2=findnumports((char**)connections2);
+    int memb1 = findnumports(connections1);
+    int memb2 = findnumports(connections2);
 
     if(memb1==0 && memb2==0)
       continue;
@@ -1393,20 +1431,20 @@ static int reconnect_ports_questionmark(void){
     if(memb1!=memb2){
       free(connections1);
       free(connections2);
-      ret=1;
-      goto exit;
+      return 1;
     }
 
     qsort(connections1,memb1,sizeof(char*),compare);
-    qsort(connections2,memb1,sizeof(char*),compare);
+    qsort(connections2,memb2,sizeof(char*),compare);
     
     {
       int lokke = 0;
       for(lokke=0;lokke<memb1;lokke++){
         //printf("connect_ports \"%s\" \"%s\" \n",connections1[lokke],connections2[lokke]);
         if(strcmp(connections1[lokke],connections2[lokke])){
-          ret=1;
-          goto exit;
+          free(connections1);
+          free(connections2);
+          return 1;
         }
       }
     }
@@ -1415,8 +1453,7 @@ static int reconnect_ports_questionmark(void){
     free(connections2);
   }
 
- exit:
-  return ret;
+  return 0;
 }
 
 
@@ -1426,7 +1463,7 @@ static void disconnect_ports(jack_port_t** ports){
   if(ports==NULL)
     return;
 
-  for(ch=0;ch<channels;ch++){
+  for(ch=0;ch<num_channels;ch++){
     int lokke = 0;
     const char **connections = jack_port_get_all_connections(client,ports[ch]);
 
@@ -1445,10 +1482,10 @@ static void connect_ports(jack_port_t** ports){
   if(ports==NULL)
     return;
 
-  for(ch=0;ch<channels;ch++){
+  for(ch=0;ch<num_channels;ch++){
     int lokke = 0;
 
-    char **connections = portnames_get_connections(ch);
+    const char **connections = portnames_get_connections(ch);
 
     while(connections && connections[lokke] != NULL){
       int err=jack_connect(client,connections[lokke],jack_port_name(ports[ch]));
@@ -1491,7 +1528,8 @@ static void* connection_thread(void *arg){
 }
 
 static void wake_up_connection_thread(void){
-  sem_post(&connection_semaphore);
+  if(use_manual_connections==false)
+    sem_post(&connection_semaphore);
 }
 
 static void start_connection_thread(void){
@@ -1505,17 +1543,22 @@ static void stop_connection_thread(void){
 }
 
 static int graphordercallback(void *arg){
-  wake_up_connection_thread();
+  if (!freewheel_mode)
+    wake_up_connection_thread();
   return 0;
 }
 
+static void freewheelcallback(int starting, void *arg){
+  freewheel_mode = starting;
+}
+
 
 
 static void create_ports(void){
-  ports = (jack_port_t **) my_calloc (sizeof (jack_port_t *),channels);  
+  ports = (jack_port_t **) my_calloc (sizeof (jack_port_t *),num_channels);  
   {
     int ch;
-    for(ch=0;ch<channels;ch++) {
+    for(ch=0;ch<num_channels;ch++) {
       char name[500];
       sprintf(name,"input%d",ch+1);
       ports[ch]=jack_port_register(client,name,JACK_DEFAULT_AUDIO_TYPE,JackPortIsInput|JackPortIsTerminal,0);
@@ -1583,8 +1626,8 @@ static pthread_t keypress_thread={0};
 static void* keypress_func(void* arg){
   char gakk[64];
   turn_off_echo();
-  fgets(gakk,49,stdin);  
-  ungetc('\n',stdin);
+  if(fgets(gakk,49,stdin) != NULL)
+    ungetc('\n',stdin);
   sem_post(&stop_sem);
   return NULL;
 }
@@ -1629,6 +1672,7 @@ static const char *advanced_help =
   "[--disable-meter] or [-dm]       -> Disable console meter.\n"
   "[--hide-buffer-usage] or [-hbu]  -> Disable buffer usage updates in the console.\n"
   "[--disable-console] or [-dc]     -> Disable console updates. Same as \"-dm -hbu\".\n"
+  "[--daemon]                       -> Don't read the console (stdin).\n"
   "[--linear-meter] or [-lm]        -> Use linear scale for the console meter (default is dB scale)\n"
   "[--dB-meter-reference or [-dBr]  -> Specify reference level for dB meter. (default=0)\n"
   "[--meterbridge] or [-mb]         -> Start up meterbridge to monitor recorded sound.\n"
@@ -1639,6 +1683,7 @@ static const char *advanced_help =
   "[--jack-transport-multi]/[-jtm]  -> Similar to --jack-transport, but do not end program when jack transport stops.\n"
   "                                    Instead, record to a new file when jack_transport starts rolling again.\n"
   "                                    (not implemented yet)\n"
+  "[--manual-connections]/[-mc]     -> jack_capture will not connect any ports for you. \n"
   "[--bufsize s] or [-B s]          -> Initial/minimum buffer size in seconds. Default is 8 seconds\n"
   "                                    for mp3 files, and 4 seconds for all other formats.\n" 
   "[--maxbufsize] or [-MB]          -> Maximum buffer size in seconds jack_capture will allocate.\n"
@@ -1696,11 +1741,11 @@ void init_arguments(int argc, char *argv[]){
                 "\n"
                 )
     {
-      OPTARG("--advanced-options","--help2") printf(advanced_help);exit(0);
+      OPTARG("--advanced-options","--help2") printf("%s",advanced_help);exit(0);
       OPTARG("--bitdepth","-b") bitdepth = OPTARG_GETINT();
       OPTARG("--bufsize","-B") min_buffer_time = OPTARG_GETFLOAT(); min_buffer_time=JC_MAX(0.01,min_buffer_time);
       OPTARG("--maxbufsize","-MB") max_buffer_time = OPTARG_GETFLOAT();
-      OPTARG("--channels","-c") channels = OPTARG_GETINT();
+      OPTARG("--channels","-c") num_channels = OPTARG_GETINT();
       OPTARG("--filename-prefix","-fp") filename_prefix = OPTARG_GETSTRING();
       OPTARG("--leading-zeros","-z") leading_zeros = OPTARG_GETINT();
       OPTARG("--recording-time","-d"){
@@ -1728,12 +1773,14 @@ void init_arguments(int argc, char *argv[]){
       OPTARG("--disable-meter","-dm") use_vu=false;
       OPTARG("--hide-buffer-usage","-hbu") show_bufferusage=false;
       OPTARG("--disable-console","-dc") use_vu=false;show_bufferusage=false;
+      OPTARG("--daemon","") no_stdin=true;
       OPTARG("--linear-meter","-lm") vu_dB=false;
       OPTARG("--dB-meter-reference","-dBr") vu_dB=true;vu_bias=powf(10.0f,OPTARG_GETFLOAT()*-0.05f);//from meterbridge
       OPTARG("--meterbridge","-mb") use_meterbridge=true;
       OPTARG("--meterbridge-type","-mt") use_meterbridge=true;meterbridge_type=OPTARG_GETSTRING();
       OPTARG("--meterbridge-reference","-mr") use_meterbridge=true;meterbridge_reference=OPTARG_GETSTRING();
       OPTARG("--jack-transport","-jt") use_jack_transport=true;
+      OPTARG("--manual-connections","-mc") use_manual_connections=true;
       OPTARG("--filename","-fn") base_filename=OPTARG_GETSTRING();
       OPTARG_LAST() base_filename=OPTARG_GETSTRING();
     }OPTARGS_END;
@@ -1756,7 +1803,7 @@ void init_arguments(int argc, char *argv[]){
 
   verbose_print("main() find default file format\n");
   if(soundfile_format_is_set==false){
-    if(channels>2)
+    if(num_channels>2)
       soundfile_format=soundfile_format_multi;
     else
       soundfile_format=soundfile_format_one_or_two;
@@ -1784,7 +1831,8 @@ void init_various(void){
   verbose_print("main() init jack 1\n");
   // Init jack 1
   {
-    start_connection_thread();
+    if(use_manual_connections==false)
+      start_connection_thread();
     start_jack();
     portnames_add_defaults();
   }
@@ -1812,7 +1860,7 @@ void init_various(void){
     sem_init(&stop_sem,0,0);
     signal(SIGINT,finish);
     signal(SIGTERM,finish);
-    if(wait_for_keyboard==true)
+    if(wait_for_keyboard==true && no_stdin==false)
       start_keypress_thread();
   }
 
@@ -1823,7 +1871,10 @@ void init_various(void){
 
     jack_on_shutdown(client, jack_shutdown, NULL);
 
-    jack_set_graph_order_callback(client,graphordercallback,NULL);
+    if(use_manual_connections==false)
+      jack_set_graph_order_callback(client,graphordercallback,NULL);
+
+    jack_set_freewheel_callback(client,freewheelcallback,NULL);
 
     if (jack_activate(client)) {
       fprintf (stderr,"\nCan not activate client");
@@ -1831,7 +1882,8 @@ void init_various(void){
     }
 
     create_ports();
-    connect_ports(ports);
+    if(use_manual_connections==false)
+      connect_ports(ports);
   }
 
 
@@ -1847,7 +1899,7 @@ void init_various(void){
   // Start meterbridge
   {
     if(use_meterbridge)
-      start_meterbridge(channels);
+      start_meterbridge(num_channels);
   }
 
   verbose_print("main() Print some info.\n");
@@ -1889,8 +1941,10 @@ void wait_until_recording_finished(void){
   if(helper_thread_running==1){
     //      if(use_vu || show_bufferusage)
     //	printf("%c[%dA",0x1b,1); // Pressing return moves the cursor.
-    if(silent==false)  // messy...
+    if(silent==false){  // messy...
       print_message("Please wait while writing all data to disk. (shouldn't take long)\n");
+      msleep(2);
+    }
     //print_message("%c[%dAPlease wait while writing all data to disk. (shouldn't take long)\n",0x1b,1); // necessary.
   }
 }
@@ -1902,7 +1956,8 @@ void stop_recording_and_cleanup(void){
 
   is_running=0;
   
-  stop_connection_thread();
+  if(use_manual_connections==false)
+    stop_connection_thread();
   
   if(jack_has_been_shut_down==false)
     jack_client_close(client);
@@ -1915,8 +1970,10 @@ void stop_recording_and_cleanup(void){
     kill(meterbridge_pid,SIGINT);
   
   stop_helper_thread();
-  
+
+
   if(silent==false){
+    usleep(50); // wait for terminal
     fprintf(stderr,"%c[31m",0x1b); //red color
     fprintf(stderr,"Finished.");
     fprintf(stderr,"%c[0m",0x1b); // reset colors
